#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Wed May 31 22:07:04 2023

@auteur: david ALBERTO
(www.astrolabe-science.fr)
Ce script trace des spectres RMN, soit d'après des valeurs expérimentales
trouvers sur internet, soit des spectres simulés.
"""
import matplotlib.pyplot as plt # graphiques
import numpy as np # listes
import pandas as pd # tableaux de données et import/export


# plt.rcParams["font.family"] = "Roboto" #ou autre police installée
plt.rcParams["font.size"] = 6

molecule = 'L-lysine'
xmin, xmax = 0.5, 4 # à ajuster aux données

NomFichieBrut = molecule + '.csv'
NomFichierExport = molecule + '_continu.csv'

prop_ligne = dict(lw=0.9, c= 'C0')# mise en forme de la ligne de la courbe

spectre = pd.read_csv(molecule+'.csv', sep=',')# import des données

ppm = spectre['ppm']
if 'mult' in spectre.columns:
    mult = spectre['mult']
H = spectre['Int.']

delta = 0.06 # écart entre les pics, pour les multiplets modélisés
a = 10000 # inverse de la largeur des pics (+ grand, + fin)

def multiplet(m, ppm, H):
    """
    m : str (multiplicité)
    H : (entier) : intensité du signal (nombre de protons)
    ppm (flottant): abscisse du signal
    renvoie une liste de valeurs Y en ordonnées
    I est l'ordonnée du pic le + faible du multiplet
    """
    if m=='d': # doublet
        I = H / 2
        Y = I * np.exp( -a*(X-ppm-delta)**2) + I * np.exp( -a*(X-ppm+delta)**2)
        return Y
    elif m=='t': # triplet
        I = H / 4
        Y = I * (np.exp( -a*(X-ppm-delta)**2) + 2 * np.exp( -a*(X-ppm)**2) + np.exp( -a*(X-ppm+delta)**2))
        return Y
    elif m=='q': # quadruplet
        I = H / 8
        Y = I * (np.exp(-a*(X-ppm-1.5*delta)**2) + 2 * np.exp(-a*(X-ppm-0.5*delta)**2) + 2 * np.exp( -a*(X-ppm+0.5*delta)**2) +  np.exp(-a*(X-ppm+1.5*delta)**2))
        return Y
    elif m=='qt': # quintuplet
        I = H / 16
        Y = I * (np.exp(-a*(X-ppm-2*delta)**2) + 4 * np.exp(-a*(X-ppm-delta)**2) + 6 * np.exp( -a*(X-ppm)**2) +  4 * np.exp(-a*(X-ppm+delta)**2) + np.exp(-a*(X-ppm+2*delta)**2))
        return Y
    elif m=='sx':# sextuplet
        I = H / 32
        Y = I * (np.exp(-a*(X-ppm-2.5*delta)**2) + 5 * np.exp(-a*(X-ppm-1.5*delta)**2) + 10 * np.exp( -a*(X-ppm-0.5*delta)**2) +  10 * np.exp(-a*(X-ppm+0.5*delta)**2) + 5 * np.exp(-a*(X-ppm+1.5*delta)**2) + np.exp(-a*(X-ppm+2.5*delta)**2))
        return Y
    else:
        Y = H*np.exp( -a*(X-ppm)**2)
        return Y

def pic_isole(ppm, H):
    """
    H : (entier) : intensité du signal (nombre de protons)
    ppm (flottant): abscisse du signal
    renvoie une liste de valeurs Y en ordonnées
    I est l'ordonnée du pic le + faible du multiplet
    """
    Y = H * np.exp( -a*(X-ppm)**2)
    return Y


def export_fichier(X,Y):
    """
    crée un DataFrame puis un fichier csv pour exporter les valeurs calculées.
    """
    df = pd.DataFrame({'ppm': X,
                        'I': Y})
    df.to_csv(NomFichierExport, index=False, sep='\t')

fig, ax = plt.subplots()

ax.invert_xaxis()
# ax.set_xlim(xmax,xmin)
ax.set_xlabel(r'$\delta$ (ppm)')
ax.set_yticks([])

X = np.arange(xmin,xmax,0.001)
Y = np.zeros_like(X)

for N in range(len(spectre)):
    x = ppm[N]
    intensite = H[N]
    if 'mult' in spectre.columns:
        m = mult[N]
        Y = Y + multiplet(m, x, intensite)
    else:
        Y = Y + pic_isole(x, intensite)

ax.plot(X,Y, **prop_ligne)
ax.text(0.01, 0.99, molecule, transform=ax.transAxes,va='top',c='firebrick')

export_fichier(X, Y)

fig.savefig('spectreRMN' + molecule + '.png',dpi=200)
fig.savefig('spectreRMN' + molecule + '.pdf')
